package Stanford::GCP::Storage;

use strict;
use warnings;
use autodie;

use Stanford::GCP::Bucket ;
use MIME::Base64;

use base qw( Exporter ) ;
our @EXPORT_OK = qw
  (
  gsutil_long_to_perl
  ) ;

our $VERSION = '0.09' ;


# Convert the output of "gsutil ls -L" to a Perl structure.
#
# {
#    'path' => 'gs:://xxxxxx',
#    'mtime_epoch' => 1556294905,
# }
sub gsutil_long_to_perl {
    my ($str) = @_ ;

    my @results = () ;

    #### # # ###     #### # # ###     #### # # ###     #### # # ###
    my $add_results_fref = sub {
        my ($current_href) = @_ ;
        if (exists($current_href->{'path'})) {
            my %tmp = %{ $current_href } ;
            my $obj = Stanford::GCP::Bucket->new() ;
            $obj->path($current_href->{'path'}) ;
            $obj->mtime_epoch($current_href->{'mtime_epoch'}) ;
            $obj->md5sum_ascii($current_href->{'md5sum_ascii'}) ;
            $obj->md5sum_base64($current_href->{'md5sum_base64'}) ;
            push(@results, $obj) ;
        } ;
    } ;
    #### # # ###     #### # # ###     #### # # ###     #### # # ###

    my %current = () ;
    my ($path, $mtime_epoch) ;
    my @lines = split(/\n/, $str) ;
    foreach my $line (@lines) {
        if ($line =~ m{^gs://}xsm) {
            # This is a first line, so save current stuff.
            $add_results_fref->(\%current) ;
            %current = () ;
            # Remove any trailing ':'
            $line =~ s{:$}{} ;

            $current{'path'} = $line ;
        } elsif ($line =~ m{^\s+goog-reserved-file-mtime:(\d+)\s*$}xsm) {
            $current{'mtime_epoch'} = $1 ;
        } elsif ($line =~ m{^\s+Hash\ \(md5\):\s*(\S+)$}ixsm) {
            my $md5hash_b64    = $1;
            my $md5hash_binary = decode_base64($md5hash_b64) ;
            my @md5hash_text   =  unpack '(H2)*', $md5hash_binary  ;
            my $md5hash_text   =  join(q{}, @md5hash_text) ;
            $current{'md5sum_ascii'}  = $md5hash_text ;
            $current{'md5sum_base64'} = $md5hash_b64 ;
        } else {
            # Do nothing
        }
    }
    $add_results_fref->(\%current) ;

    return @results ;
}


1;
